#' Calculate the variation of site/genes amount and shows the results for each tool.
#'
#' @param all.sites.methods data.frame produced by pre_process or get_TCGA_sites or site_annotation functions
#' @param site All/coding/noncoding
#' @param type Site or Gene
#' @param bin number of points into which the explored interval is discretized
#' @param fileSetting configuration file name
#' @return list with the following elements:
#' \itemize{
#' \item{for each parameter a matrix with site amount at varying parameter values in each tool;}
#' \item{matrix with settings.}
#' }
#' @importFrom utils read.table
#' @export


ese_allsubj <- function(all.sites.methods, site = "All", type = NULL , bin = NULL, fileSetting = NULL) {

  if(is.null(type)){
    stop("You have to define type. The options are: \"Site\" or \"Gene\".")
  }
  if(type=="Gene" & !"Gene" %in% colnames(all.sites.methods)){
    stop("The sites are not annotated! You should use: \"type = Site\"")
  }
  if(nrow(all.sites.methods)==0){
    stop("The dataframe with the sites is empty.")
  } else if (nrow(all.sites.methods)>0 & type=="Site"){
    all.sites.methods <- all.sites.methods[!duplicated(all.sites.methods[,c('CHROM', 'POS', 'REF', 'ALT', 'SiteID_Subject', 'Tool_ID')]), ]
  }
  #For analyzing only coding mutation sites
  if(site %in% c("coding", "noncoding") & !"Location" %in% colnames(all.sites.methods)){
    stop("The sites are not annotated! You should use \"site = All\"")
  } else if(site == FALSE) {stop("Don't you FALSE, you can use: coding, noncoding or using All for all sites.")}

  if(site!="All"){
    switch(site,
           coding ={all.sites.methods <- all.sites.methods[which(all.sites.methods$Location %in% "coding"), ]
           nrowSitesNew <- nrow(all.sites.methods)
           if(nrowSitesNew==0) {stop("No coding sites")}
           },
           noncoding ={   all.sites.methods <- all.sites.methods[which(all.sites.methods$Location %in% "noncoding"), ]
           nrowSitesNew <- nrow(all.sites.methods)
           if(nrowSitesNew==0) {stop("No noncoding sites")}
           },
           {
             stop(sprintf(paste("Wrong type of site selected: ", site, ". You can use: coding, noncoding or using All for all sites.")))
           }
    )
  }

  if(is.null(fileSetting)){
    parameter <- c("Tumor_var_reads",
                   "Normal_var_reads",
                   "Tumor_Vaf",
                   "Normal_Vaf",
                   "Tumor_reads",
                   "Normal_reads"
    )
    minValue <- c(min(all.sites.methods[,"Tumor_var_reads"]),
                  min(all.sites.methods[,"Normal_var_reads"]),
                  min(all.sites.methods[,"Tumor_Vaf"]),
                  min(all.sites.methods[,"Normal_Vaf"]),
                  min(all.sites.methods[,"Tumor_reads"]),
                  min(all.sites.methods[,"Normal_reads"])
    )
    maxValue <- c(max(all.sites.methods[,"Tumor_var_reads"]),
                  max(all.sites.methods[,"Normal_var_reads"]),
                  max(all.sites.methods[,"Tumor_Vaf"]),
                  max(all.sites.methods[,"Normal_Vaf"]),
                  max(all.sites.methods[,"Tumor_reads"]),
                  max(all.sites.methods[,"Normal_reads"])
    )
    operator <- c(">=",
                  "<=",
                  ">=",
                  "<=",
                  ">=",
                  ">="
    )
    settingParameter <- data.frame(parameter,
                                   minValue,
                                   maxValue,
                                   operator,
                                   stringsAsFactors = FALSE
    )
  } else {settingParameter <- read.table(fileSetting, header = TRUE, stringsAsFactors = FALSE, sep = "\t")
  }

  if(is.null(fileSetting) & is.null(bin)){
    settingParameter$bin <- rep(20, nrow(settingParameter))
  } else if(is.null(fileSetting) & !is.null(bin)){
    settingParameter$bin <- rep(bin, nrow(settingParameter))
  }
  logicColumnPresence <- unlist(apply(all.sites.methods[as.character(settingParameter$parameter)], 2, function(x) any(!is.na(x))))
  settingParameter <- settingParameter[logicColumnPresence, ]
  if(nrow(settingParameter)==0){
    stop("The columns in the dataframe of the selected parameters have \"NA\" values")
  }
  #Unique tool-ID and Subject-Id
  Tool_ID <- unique(all.sites.methods$Tool_ID)
  Subject_ID <- unique(all.sites.methods$Subject_ID)
  #List for save the data.frame for each parameter
  filteringOutputByTool <- vector('list', nrow(settingParameter))
  for (j in 1:nrow(settingParameter)) {
    #Extract the column of the parameter to calculate the min and the max global
    parameter <- settingParameter[j, "parameter"]
    operator <- settingParameter[j, "operator"]
    #outByTool: for analyzing the variation of the amount of mutations sites for
    #each tool
    outByTool <- matrix(data = NA,
                        nrow = as.numeric(settingParameter[j,"bin"]),
                        ncol = length(Tool_ID)
    )
    colnames(outByTool) <- Tool_ID
    outByTool <- data.frame(par = 0,
                            outByTool,
                            stringsAsFactors = F
    )
    #Generate a sequence of number that start from the min value and the max
    #value calculted before. The lenght of the sequence is chosen by "bin"
    outByTool$par <- seq(from = as.numeric(settingParameter[j,"minValue"]),
                         to = as.numeric(settingParameter[j, "maxValue"]),
                         length.out = as.numeric(settingParameter[j,"bin"])
    )
    for (i in 1:length(Tool_ID)) {
      #Extract all mutation sites of the Tool_ID[i]
      tempTool <- all.sites.methods[all.sites.methods$Tool_ID == Tool_ID[i], ]
      #Switch into the type selected analysis
      switch(type,
             "Site"={
               switch(operator,
                      "<="={ outByTool[, i + 1] <- sapply(outByTool$par,
                                                          function(x) nrow(tempTool[abs(tempTool[parameter]) <= x, ]))
                      },
                      ">="={ outByTool[, i + 1] <- sapply(outByTool$par,
                                                          function(x) nrow(tempTool[abs(tempTool[parameter]) >= x, ]))
                      },
                      "=="={ outByTool[, i + 1] <- sapply(outByTool$par,
                                                          function(x) nrow(tempTool[abs(tempTool[parameter]) == x, ]))
                      },
                      "!="={ outByTool[, i + 1] <- sapply(outByTool$par,
                                                          function(x) nrow(tempTool[abs(tempTool[parameter]) != x, ])) },
                      ">"={ outByTool[, i + 1] <- sapply(outByTool$par,
                                                         function(x) nrow(tempTool[abs(tempTool[parameter]) > x, ]))  },
                      "<"={ outByTool[, i + 1] <- sapply(outByTool$par,
                                                         function(x) nrow(tempTool[abs(tempTool[parameter]) < x, ]))     },
                      { stop(sprintf(paste("Wrong Operator: ", operator, ", please use: <=, <, ==, >, >=, !=" )))
                      }
               )
             },
             "Gene"={
               switch(operator,
                      "<="={ outByTool[, i + 1] <- sapply(outByTool$par,
                                                          function(x) length(unique(tempTool[abs(tempTool[parameter]) <= x, "Gene" ])))
                      },
                      ">="={ outByTool[, i + 1] <- sapply(outByTool$par,
                                                          function(x) length(unique(tempTool[abs(tempTool[parameter]) >= x, "Gene" ])))
                      },
                      "=="={ outByTool[, i + 1] <- sapply(outByTool$par,
                                                          function(x) length(unique(tempTool[abs(tempTool[parameter]) == x, "Gene" ])))
                      },
                      "!="={ outByTool[, i + 1] <- sapply(outByTool$par,
                                                          function(x) length(unique(tempTool[abs(tempTool[parameter]) != x, "Gene" ])))
                      },
                      ">"={ outByTool[, i + 1] <- sapply(outByTool$par,
                                                         function(x) length(unique(tempTool[abs(tempTool[parameter]) > x, "Gene" ])))
                      },
                      "<"={ outByTool[, i + 1] <- sapply(outByTool$par,
                                                         function(x) length(unique(tempTool[abs(tempTool[parameter]) < x, "Gene" ])))  },
                      { stop(sprintf(paste("Wrong Operator: ", operator, ", please use: <=, <, ==, >, >=, !=" )))
                      })
             }, {stop(sprintf(paste("Wrong type: ", type, ", please use: Site or Gene.")))
             }
      )
      filteringOutputByTool[[j]] <- outByTool
    }
  }
  #Rename by the parameter
  names(filteringOutputByTool) <- settingParameter$parameter
  listOutput <- list(filtering_output = filteringOutputByTool, setting_filtering = settingParameter)
  return(listOutput)
}
